/**************************************************************************
* AVR FFT displayed on Nokia 3310 LCD
*
* copyright 2009 Michael Spiceland (http://tinkerish.com)
*
***************************************************************************
**
**  This program is free software; you can redistribute it and/or
**  modify it under the terms of the GNU General Public License
**  as published by the Free Software Foundation; either version 2
**  of the License, or (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software Foundation, 
**  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
**
**************************************************************************/
#define F_CPU 8000000
#define BAUD_RATE     38400//9600            // desired baud rate
#define UBRR_DATA     (F_CPU/(BAUD_RATE)-1)/16  // sets baud rate

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <avr/sleep.h>
#include <stdio.h>
#include <inttypes.h>
#include <ctype.h>
#include <util/delay.h>
#include <math.h>
#include "ffft.h"
#include "3310_routines.h"

/* defines */
#define ALL_INPUT  0x00  //0000 0000
#define ALL_OUTPUT 0xFF  //1111 1111

int16_t capture[FFT_N];			/* Wave captureing buffer */
complex_t bfly_buff[FFT_N];		/* FFT buffer */
uint16_t spektrum[FFT_N/2];		/* Spectrum output buffer */

#define NUM_SAMPLES  FFT_N
#define FFT_SIZE FFT_N / 2

/************************************************************************
 * SIG_OVERFLOW0
 *
 * This routine is run evertime the timer counter overflows.  Since the
 * timer counter is 8 bits, it can count to 255 before overflowing and
 * going back to 0.  The frequency this is executed in is determined by
 * the clock speed / clk_divide (TCCR0) / 256.
 ***********************************************************************/
SIGNAL(SIG_OVERFLOW0) {
	static uint16_t count = 0;
	static uint16_t offset=0;
	uint8_t adc_val;

	/* we only want to run every clk/4 times  8khz */
	count++;
	if ((count % 2)) // 2 = 16mhz, 4 = 8mhz, 8 = 4mhz
		return;

	ADCSRA |= (1 << ADSC); // start ADC conversion
	while (ADCSRA & (1 << ADSC)) {;}; // wait for the result to be available
	adc_val = ADCH;
	
	capture[offset] = (int16_t)(adc_val)-127; // convert to signed linear
	offset++;

	if (offset == NUM_SAMPLES) // buffer is full
	{
		fft_input(capture, bfly_buff);
		fft_execute(bfly_buff);
		fft_output(bfly_buff, spektrum);
		display_fft(spektrum, NUM_SAMPLES/2);

		offset = 0;
	}
}

int main(void){
	
	/* initialize port data directions */
	DDRB = ALL_OUTPUT; // SPI for LCD
	PORTB = 0xFF;
	DDRC &= ~(1<<0); //AIN0 is input
	PORTC &= ~(1<<0); //AIN0 is input pullup off
	DDRD &= ~(1<<1); // UART RX
	DDRD |= (1<<1); // UART TX


	/* initialize timers */
	TCCR0 |=  _BV(CS00); // clk prescale (clk/1)
	TIMSK |= _BV(TOIE0); // enable timer/counter0 overflow interrupt 
	TCNT0 = 0;           // reset the timer

	/* initialize the ADC */
	ADMUX |= _BV(ADLAR); // we only want 8-bit
	ADCSRA |= _BV(ADEN); // for now we don't do this in the ISR | _BV(ADIE);
	ADCSRA |= _BV(ADPS2); // /16 | _BV(ADPS0); // clk is / 32 

	spi_init();

	/* get LCD ready */
	LCD_init();
	_delay_loop_2(65535);
	LCD_clear();
	LCD_gotoXY(4,1);
	LCD_writeString_F("FFT");
	LCD_gotoXY(4,5);
	LCD_writeString_F("by mspice");
	
	sei(); //enable global interrupts

	/* AVR's like to have a loop */
	while(1) {
	}
} 

